//
// bignum_type_traits.inc
//
// Bill Seymour, 2022-02-23
//
// Copyright Bill Seymour 2022.
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//
// This is not a proper header file.  It gets #included in the private parts
// of the bigint and bigdec classes defined in bigint.hpp and bigdec.hpp.
// Both of those classes have implicit ctors that allow conversion from
// fundamental arighmetic types except bool, and use the following
// to exclude bool.  Both headers have already #included <type_traits>.
//

template<typename T>
using is_bool = std::is_same<typename std::remove_cv<T>::type, bool>;

template<typename T> using is_flt = std::is_floating_point<T>;

template<typename T>
struct is_sgn : std::integral_constant<bool,
    std::is_signed<T>::value && !is_flt<T>::value> { };

template<typename T>
struct is_uns : std::integral_constant<bool,
    std::is_unsigned<T>::value && !is_bool<T>::value> { };

template<typename T>
struct is_ari : std::integral_constant<bool,
    std::is_arithmetic<T>::value && !is_bool<T>::value> { };

// End of bignum_type_traits.inc
